﻿/*
AwesomeNET by xeetsh - Maximilian T. Schmitt

Licensed under the Unity Asset Store Terms of Service and EULA. See https://unity3d.com/legal/as_terms for a complete version of the license

AwesomeNET Doccumentation: https://docs.awesomenet.xeet.sh
Contact/Support/Feature requests: https://xeet.sh/contact.php
Twitter: @xeetsh
*/

using System.Collections;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System;
using UnityEngine;

public class MatchmakingClient : TCPClient
{
    public MatchmakingClient(string host, int port) : base(host, port) {
        
    }

    public string[] GetAdvertisements() {
        Send((int) CC.GetAdvertisements);
        if (AnalyzeMessage(Receive()) == (int) CCType.GameList) {
            return m_lastArgv;
        }
        PrintMessage("Received something that isn't a game list", MessageTypes.Error);
        return null;
    }

    public string AdvertiseNewGame(string map, string player) {
        Send((int) CC.AdvertiseNewGame, new string[] {map, player});
        if (AnalyzeMessage(Receive()) == (int) CCStatus.Success) {
            PrintMessage(string.Format("Advertised new game on {0} as {1}. ID: {2}", map, player, m_lastArgv[0]), MessageTypes.Notification);
            return m_lastArgv[0];
        } else {
            PrintMessage("Error while advertising game", MessageTypes.Error);
            return null;
        }
    }

    public IPEndPoint JoinGame(string id, string player) {
        Send((int) CC.JoinGame, new string[] {player, id});
        if (AnalyzeMessage(Receive()) == (int) CCStatus.Success) {
            PrintMessage(string.Format("Joined game {0} on {1}:{2}", id, m_lastArgv[0], m_lastArgv[1]), MessageTypes.Notification);
            return new IPEndPoint(Dns.GetHostAddresses(m_lastArgv[0])[0], int.Parse(m_lastArgv[1]));
        } else {
            PrintMessage("Error while advertising game", MessageTypes.Error);
            return null;
        }
    }

    public GameInfo GetGameInfo(string id) {
        Send((int) CC.GetGameInfo, new string[] {id});
        GameInfo gi = new GameInfo();
        if (AnalyzeMessage(Receive()) == (int) CCStatus.Success) {
            gi.playercount = int.Parse(m_lastArgv[0]);
            gi.host = m_lastArgv[1];
            gi.map = m_lastArgv[2];
            gi.players = m_lastArgv[3].Split(';');
            return gi;
        } else {
            PrintMessage("Couldnt retreive game info", MessageTypes.Error);
            return gi;
        }
    }

    public override void OnConnect() {
        PrintMessage(string.Format("Connected to matchmaking server on {0}:{1}", m_host, m_port), MessageTypes.Notification);
    }
    public override void OnDisconnect() {
        PrintMessage("Disconnected from matchmaking server", MessageTypes.Notification);
    }

    public override string GetServerType(bool uppercase) {
        if (uppercase)
            return "Matchmaking";
        else
            return "matchmaking";
    }
}

public struct GameInfo {
    public string host;
    public int playercount;
    public string[] players;
    public string map;
}

